import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;

public class FastaAutomat{
  int state = 0;

  /** Zwei Hilfsmethoden, die das Testen von 
    Basen und alphanumerischen Zeichen erleichtern sollen. **/
  boolean isBase( char c ){
    return c == 'a' || c == 'c' || c == 'g' || c == 't' || c == '.';
  }
  boolean isLetter( char c ){
    return c != '\n' && c != '|';
  }

  /** Diese Methode simuliert die Zustandsübergänge des zugrunde
      liegenden endlichen Automaten. **/
  void handleInput(char c){
    /** 
     *
     *
     * Hier soll der eigene Code hin.
     *
     *
     */
    if( state == 0 ){
      // Startzustand 
      if( c == '>' ) state = 1;
      else state = -1; // ERROR
    }
    else if( state == 1 ){
      // Erwarte Sequenzheader
      if( isLetter( c ) ) state = 2;
      else state = -1;
    }
    else if( state == 2 ){
      // Sequenzheader begonnen
      if( isLetter( c ) ) state = 2;
      else if( c == '|' ) state = 1;
      else if( c == '\n' ) state = 3;
    }
    else if( state == 3 ){
      // Erwarte Sequenz oder Kommentar
      if( isBase( c ) ) state = 5;
      else if( c == ';' ) state = 4;
      else state = -1;
    }
    else if( state == 4 ){
      // Kommentar begonnen
      if( isLetter( c ) || c == '|' ) state = 4;
      else if( c == '\n' ) state = 3;
    }
    else if( state == 5 ){
      // Sequenz begonnen (akzeptierend)
      if( isBase( c ) ) state = 5;
      else if( c == '\n' ) state = 6;
      else state = -1;
    }
    else if( state == 6 ){
      // Sequenzzeile beendet (akzeptierend)
      if( isBase( c ) ) state = 5;
      else if( c == '>' ) state = 1;
      else state = -1;
    }
  }

  /** Diese Methode entscheidet, ob ein kompletter String vom 
      Automaten akzeptiert wird. **/
  boolean isAccepted(String s){
    this.state = 0;
    for(int i = 0 ; i < s.length(); i++){
	    handleInput(s.charAt(i));
    }
    return state == 5 || state == 6; // +++ eventuell anpassen +++
  }
  
  String readInput(String filename) {
    StringBuilder sb = new StringBuilder();
    try {
      FileInputStream fstream = new FileInputStream(filename);
      DataInputStream in = new DataInputStream(fstream);
      BufferedReader br = new BufferedReader(new InputStreamReader(in));
      String line;
      while ((line = br.readLine()) != null) {
        sb.append(line + '\n');
      }
      br.close();
      in.close();
      fstream.close();
    } catch (Exception e) {
      System.err.println("Fehler beim Öffnen der Datei: " + e.getMessage());
      System.err.println("Ist die Datei im richtigen Ordner? "
              + "Ist der Dateiname richtig geschrieben?");
    }
    return sb.toString();
  }

  public static void main(String[] args){
      // Automat-Objekt erstellen, damit die nichtstatischen Methoden 
      // aufgerufen werden können
      FastaAutomat a = new FastaAutomat();
      // Lesen der Eingabe
      String input = a.readInput(args[0]);
      // Eingabevalidierung durchführen
      System.out.println( "Eingabe Akzeptiert: "+a.isAccepted(input));
  }
}
