import greenfoot.Greenfoot; // We import the Greenfoot namespace for the random number generator.

import java.lang.Math;

/* This class implements a brain which will execute the following actions in that order: 
 -- TODO
*/

public class StupidBrain extends Brain {

    private int direction = EAST;

    public void init() {
        this.creature.setCreatureImage("wombat.png");
    }

    public void think() {
        // First, we look a the current field to see if there is something to
        // eat here.
        // Looking at the field where the creature stands does not cost
        // anything, so this is generally a good idea.
        Thing[] here = this.creature.lookAround(0);
        boolean somethingToEat = false;
        for (Thing t : here) {
            if (t.getType() == Thing.PLANT) {
                somethingToEat = true;
            }
        }

        if (somethingToEat) {
            // If we have found something to eat on the current field, our
            // action in this turn is simply to eat.
            // Since we can do only one action per turn, we do not do anything
            // else.
            this.creature.say("Eat");
            this.creature.eat();
        } else {
            // Otherwise we look at the environment. This costs our creature
            // energy, according to the specified range.
            // Here we always look the furthest we can, even though this costs
            // the most.
            int range = this.creature.getMaxViewRange();
            Thing[] env = this.creature.lookAround(range);

            // We walk towards the center of the field.
            this.creature.say("Walk to the center");
            int worldHeight = this.creature.getWorldHeight();
            int worldWidth = this.creature.getWorldWidth();
            if (this.creature.getY() < worldHeight/2) {
                this.direction = SOUTH;
            } else if (this.creature.getY() > worldHeight/2) {
               this.direction = NORTH;
            } else if (this.creature.getX() < worldWidth/2) {
                this.direction = EAST;
            } else {
                this.direction = NONE;
            }
            

            // Finally, we move the creature to the previously computed
            // position.
            if (this.direction != NONE)
                this.creature.move(this.direction);
        }
    }

    private double distance(int ax, int ay, int bx, int by) {
        int dx = ax - bx;
        int dy = ay - by;
        return Math.sqrt(dx * dx + dy * dy);
    }

    public String getName() {
        return "WombatBrain";
    }

}
