import greenfoot.Greenfoot; // We import the Greenfoot namespace for the random number generator.

import java.lang.Math;

/* This class implements a brain which will execute the following actions in that order: 
If you are standing on a plant: eat it. 
If you have less than half energy: Find a plant and eat it.
If there is an enemy close by (less than 4 spaces away): run from it. 
If there is a plant nearby: walk towards it.
Else: Walk to the middle 
ALWAYS check if there is an obstaclre in the way first.
*/

public class ExampleBrain extends Brain {

    private int direction = EAST;
    private int counter = 1;

    public void init() {
        this.creature.setCreatureImage("tux.png");
    }

    public void think() {
		int former_direction = this.direction;
        /* First, we look a the current field to see if there is something to
        eat here.
        Looking at the field where the creature stands does not cost
        anything, so this is generally a good idea. */
        Thing[] here = this.creature.lookAround(0);
        boolean somethingToEat = false;
        for (Thing t : here) {
            if (t.getType() == Thing.PLANT) {
                somethingToEat = true;
            }
        }
        counter++;

        if (somethingToEat) {
            /* If we have found something to eat on the current field, our
             action in this turn is simply to eat.
            Since we can do only one action per turn, we do not do anything
            else. */
            this.creature.say("Eat");
            this.creature.eat();
        } else {
            /* Otherwise we look at the environment. This costs our creature
             energy, according to the specified range.
             Here we always look the furthest we can, even though this costs
             the most. */
            int range = this.creature.getMaxViewRange();
            Thing[] env = this.creature.lookAround(range);

            /* Then we try to find the nearest plant from the list of things we
             can see. */
            Thing nearestPlant = null;
            Thing closeEnemy = null;
            double plantDist = Double.POSITIVE_INFINITY;
            double enemyDist = 4;
            for (Thing t : env) {
                if (t.getType() == Thing.BEAR) {
                    double tmp = this.distance(this.creature.getX(), this.creature.getY(), t.getX(), t.getY());
                    if (tmp < enemyDist) {
                        enemyDist = tmp;
                        closeEnemy = t;
                    }
                } 
                if (t.getType() == Thing.PLANT) {
                    double tmp = this.distance(this.creature.getX(), this.creature.getY(), t.getX(), t.getY());
                    if (tmp < plantDist) {
                        plantDist = tmp;
                        nearestPlant = t;
                    }
                }
            }

            if (closeEnemy != null && this.creature.getEnergy() > 100){
                /* If we see an enemy somewhere around us, we change our moving
                direction and run from it if we have enough energy. */
                boolean west = false;
                boolean east = false;
                boolean south = false;
                boolean north = false;
                this.creature.say("Run from bear");
                if (this.creature.getX() < closeEnemy.getX()) {
                    west = true;
                    this.direction = WEST;
                } 
				if (this.creature.getX() > closeEnemy.getX()) {
                    east = true;
                    this.direction = EAST;
                } 
				if (this.creature.getY() < closeEnemy.getY()) {
                    north = true;
                    this.direction = NORTH;
                } 
				if (this.creature.getY() > closeEnemy.getY()){
                    south = true;
                    this.direction = SOUTH;
                }                  

            } else if (nearestPlant != null) {
                /* If we see a plant somewhere around us, we change our moving
                 direction and run towards it. */
                this.creature.say("Walk to food");
                if (this.creature.getX() < nearestPlant.getX()) {
                    this.direction = EAST;
                } else if (this.creature.getX() > nearestPlant.getX()) {
                    this.direction = WEST;
                } else if (this.creature.getY() < nearestPlant.getY()) {
                    this.direction = SOUTH;
                } else {
                    this.direction = NORTH;
                }
            } else {
                 /* If we do not see a plant or bear somewhere, we walk towards the middle of the field.*/
                this.creature.say("Walk to the middle");
                int worldHeight = this.creature.getWorldHeight();
                int worldWidth = this.creature.getWorldWidth();
                if (this.creature.getY() < worldHeight/2) {
                    this.direction = SOUTH;
                } else if (this.creature.getY() > worldHeight/2) {
                    this.direction = NORTH;
                } else if (this.creature.getX() < worldWidth/2) {
                    this.direction = EAST;
                } else {
                    this.direction = WEST;
                }
            }

			/*before we move, we try not to run into a wall.*/
			boolean eastWall = false;
			boolean westWall = false;
			boolean northWall = false;
			boolean southWall = false;
			/* Check for walls in adjacing fields */
			for (Thing t : env) {
                if (t.getType() == Thing.WATER || t.getType() == Thing.ROCK) {
                    double tmp = this.distance(this.creature.getX(), this.creature.getY(), t.getX(), t.getY());
                    if (tmp == 1) {
                        if (t.getX() == this.creature.getX() - 1){
							westWall = true;
						} else if (t.getX() == this.creature.getX() + 1){
							eastWall = true;
						} else if (t.getY() == this.creature.getY() -1 ){
							northWall = true;
						} else {
							southWall = true;
						}
                    }
                } 
            }
			/* If there is a wall, turn untill you have a free way*/
			if (eastWall || westWall || northWall || southWall){
			    boolean wayFree = false;
				while (!wayFree){
					if (this.direction == EAST && eastWall){
						if (former_direction != NORTH){
							this.direction = NORTH;
						} else {
							this.direction = SOUTH; 
						}
					} else if (this.direction == WEST && westWall){
						if (former_direction != SOUTH){
							this.direction = SOUTH;
						} else {
							this.direction = NORTH; 
						}
					} else if (this.direction == NORTH && northWall){
						if (former_direction != WEST){
							this.direction = WEST;
						} else {
							this.direction = EAST; 
						}
					} else {
						if (former_direction != EAST){
							this.direction = EAST;
						} else {
							this.direction = WEST; 
						}
					}	
					wayFree = true;
					/*check if we are not headed for a new obstacle*/
					if ((this.direction == EAST && eastWall) || (this.direction == WEST && westWall) || (this.direction == NORTH && northWall) || (this.direction == SOUTH && southWall)){
						wayFree = false;
					} 
				}
			}
            /* Finally, we move the creature to the previously calculated
             position.*/
            this.creature.move(this.direction);
        }
    }

    private double distance(int ax, int ay, int bx, int by) {
        int dx = ax - bx;
        int dy = ay - by;
        return Math.sqrt(dx * dx + dy * dy);
    }

    public String getName() {
        return "Example Brain";
    }

}
