import greenfoot.Greenfoot;

import java.lang.Math;


public class ExampleBrain2 extends Brain {

    private int direction = EAST;

    public void init() {
        this.creature.setCreatureImage("frog.png");
    }

    public void think() {
        // First, we look a the current field to see if there is something to
        // eat here.
        // Looking at the field where the creature stands does not cost
        // anything, so this is generally a good idea.
        Thing[] here = this.creature.lookAround(0);
        Thing eatThis = null;
        Thing attackThis = null;
        for (Thing t : here) {
            if (t.getType() == Thing.PLANT || t.getEnergy() <= 0) {
                eatThis = t;
            } else if (t.getType() == Thing.MOUSE) {
                attackThis = t;
            }
        }

        if (this.creature.getEnergy() > 190) {
            this.creature.giveBirth();
            return;
        } else if(attackThis != null) {
            this.creature.attack(attackThis);
            return;
        } else if (this.creature.getEnergy() < 150 && eatThis != null) {
            // If we have found something to eat on the current field, our
            // action in this turn is simply to eat.
            // Since we can do only one action per turn, we do not do anything
            // else.
            this.creature.eat();
            return;
        } else {
            // Otherwise we look at the environment. This costs our creature
            // energy, according to the specified range.
            // Here we always look the furthest we can, even though this costs
            // the most.
            int range = this.creature.getMaxViewRange();
            Thing[] env = this.creature.lookAround(range);

            // Then we try to find the nearest plant from the list of things we
            // can see.
            Thing nearestThing = null;
            double dist = Double.POSITIVE_INFINITY;
            for (Thing t : env) {
                if (t.getType() == Thing.PLANT || t.getType() == Thing.MOUSE) {
                    double tmp = this.distance(this.creature.getX(), this.creature.getY(), t.getX(), t.getY());
                    if (tmp < dist) {
                        dist = tmp;
                        nearestThing = t;
                    }
                }
            }

            if (nearestThing != null) {
                if(nearestThing.getType() == Thing.MOUSE && nearestThing.getEnergy() > 0 && dist <= 1) {
                    this.creature.attack(nearestThing);
                    return;
                }
                // Otherwise change our moving direction.
                if (this.creature.getX() < nearestThing.getX()) {
                    this.direction = EAST;
                } else if (this.creature.getX() > nearestThing.getX()) {
                    this.direction = WEST;
                } else if (this.creature.getY() < nearestThing.getY()) {
                    this.direction = SOUTH;
                } else {
                    this.direction = NORTH;
                }
            } else {
                // If we do not see a plant somewhere, we either leave the
                // direction unchanged or choose a new random direction
                if (Greenfoot.getRandomNumber(4) < 1) {
                    int newDir = Greenfoot.getRandomNumber(4);
                    if (newDir == 0) {
                        this.direction = EAST;
                    } else if (newDir == 1) {
                        this.direction = NORTH;
                    } else if (newDir == 2) {
                        this.direction = WEST;
                    } else {
                        this.direction = SOUTH;
                    }
                }
            }

            // Finally, we move the creature to the previously calculated
            // position.
            this.creature.move(this.direction);
        }
    }

    private double distance(int ax, int ay, int bx, int by) {
        int dx = ax - bx;
        int dy = ay - by;
        return Math.sqrt(dx * dx + dy * dy);
    }

    public String getName() {
        return "Example Brain 2";
    }

}
