import infoatools.SimpleIO;

public class FastaAutomat{
  int state = 0;

  /** Zwei Hilfsmethoden, die das Testen von 
    Basen und alphanumerischen Zeichen erleichtern sollen. **/
  boolean isBase( char c ){
    return c == 'a' || c == 'c' || c == 'g' || c == 't' || c == '.';
  }
  boolean isLetter( char c ){
    return c != '\n' && c != '|';
  }

  /** Diese Methode simuliert die Zustandsübergänge des zugrunde
      liegenden endlichen Automaten. **/
  void handleInput( char c ){
    if( state == 0 ){
      // Startzustand 
      if( c == '>' ) state = 1;
      else state = -1; // ERROR
    }
    else if( state == 1 ){
      // Erwarte Sequenzheader
      if( isLetter( c ) ) state = 2;
      else state = -1;
    }
    else if( state == 2 ){
      // Sequenzheader begonnen
      if( isLetter( c ) ) state = 2;
      else if( c == '|' ) state = 1;
      else if( c == '\n' ) state = 3;
    }
    else if( state == 3 ){
      // Erwarte Sequenz oder Kommentar
      if( isBase( c ) ) state = 5;
      else if( c == ';' ) state = 4;
      else state = -1;
    }
    else if( state == 4 ){
      // Kommentar begonnen
      if( isLetter( c ) || c == '|' ) state = 4;
      else if( c == '\n' ) state = 3;
    }
    else if( state == 5 ){
      // Sequenz begonnen (akzeptierend)
      if( isBase( c ) ) state = 5;
      else if( c == '\n' ) state = 6;
      else state = -1;
    }
    else if( state == 6 ){
      // Sequenzzeile beendet (akzeptierend)
      if( isBase( c ) ) state = 5;
      else if( c == '>' ) state = 1;
      else state = -1;
    }
  }

  /** Diese Methode entscheidet, ob ein kompletter String vom 
      Automaten akzeptiert wird. **/
  boolean isAccepted( String s ){
      this.state = 0;
      for( int i = 0 ; i < s.length() ; i ++ ){
	 handleInput( s.charAt( i ) );
      }
      return state == 5 || state == 6;
  }

  public static void main( String[] args ){
      // SimpleIO benutzen, um die übergebene Datei zu lesen
      String[] inhalt = SimpleIO.readFile( args[0] );
      // Zeilen der Datei zu einem String zusammenfügen
      String eingabe = "";
      for( int i = 0 ; i < inhalt.length ; i ++ ){
	  eingabe = eingabe + inhalt[i] + '\n';
      }
      // Automat-Objekt erstellen, damit die nichtstatischen
      // Methoden aufgerufen werden können
      FastaAutomat a = new FastaAutomat();
      // Eingabevalidierung durchführen
      System.out.println( "Eingabe Akzeptiert: "+a.isAccepted( eingabe ) );
  }
}
