import java.sql.*;
import java.util.regex.*;

class Datenbestand {

    class Autor {
	int id;
	String name;
    }

    class Artikel {
	int id;
	String titel;
	Zeitschrift erschienenIn;
    }

    class Zeitschrift {
	int id;
	String titel;
    }

    Autor[]       alleAutoren;
    Artikel[]     alleArtikel;
    Zeitschrift[] alleZeitschriften;


    Connection c;
    Statement s;
    ResultSet results;
    
    void createConnection (String user, String password) throws Exception {
	Class.forName("com.mysql.jdbc.Driver");  
	c = DriverManager.getConnection
	    ("jdbc:mysql://localhost/biblio", user, password);
	s = c.createStatement ();
    }

    void readAllAuthors () throws Exception {
	s.executeQuery ("select count(*) as n from autor");
	results = s.getResultSet ();
	results.next();
	int n = results.getInt("n");

	alleAutoren = new Autor [n];
	s.executeQuery ("select * from autor");
	results = s.getResultSet ();
	int i = 0;
	while (results.next()) {
	    alleAutoren[i] = new Autor ();
	    alleAutoren[i].id = results.getInt("id");
	    alleAutoren[i].name = results.getString("name");
	    i++;
	}
    }

    void readAllArticles () throws Exception {
	s.executeQuery ("select count(*) as n from artikel");
	results = s.getResultSet ();
	results.next();
	int n = results.getInt("n");
	
	alleArtikel = new Artikel [n];
	
	s.executeQuery ("select * from artikel");
	results = s.getResultSet ();
	int i = 0;
	while (results.next()) {
	    alleArtikel[i] = new Artikel ();
	    alleArtikel[i].id = results.getInt("id");
	    alleArtikel[i].titel = results.getString("titel");
	    i++;
	}
    }

    void readAllJournals () throws Exception {
	s.executeQuery ("select count(*) as n from zeitschrift");
	results = s.getResultSet ();
	results.next();
	int n = results.getInt("n");
	
	alleZeitschriften = new Zeitschrift [n];
	
	s.executeQuery ("select * from zeitschrift");
	results = s.getResultSet ();
	int i = 0;
	while (results.next()) {
	    alleZeitschriften[i] = new Zeitschrift ();
	    alleZeitschriften[i].id = results.getInt("id");
	    alleZeitschriften[i].titel = results.getString("titel");
	    i++;
	}
    }

    void readEnthaeltTable () throws Exception {
	s.executeQuery ("select * from enthaelt");
	results = s.getResultSet ();
	while (results.next()) {
	    int artikelId = results.getInt("artikel_id");
	    int zeitschriftId = results.getInt("zeitschrift_id");
	    // Finde Zeitschrift:
	    Zeitschrift z = null;
	    for (int i = 0; i < alleZeitschriften.length; i++)
		if (alleZeitschriften[i].id == zeitschriftId)
		    z = alleZeitschriften[i];
	    // Finde Artikel:
	    Artikel a = null;
	    for (int i = 0; i < alleArtikel.length; i++)
		if (alleArtikel[i].id == artikelId)
		    alleArtikel[i].erschienenIn = z;
	}
    }



    String extractTitel (String bibtexEintrag) {
	 Pattern p = Pattern.compile("title *= *\\{(.*?)\\}");
	 Matcher m = p.matcher(bibtexEintrag);
	 if (m.find ()) {
	     return m.group (1);
	 } else {
	     return null;
	 }
    }

    void insertArticleIntoDatabse (String bibtexEintrag) throws Exception {
	// Extrahiere den Artikelname:	
	String title = extractTitel (bibtexEintrag);

	// Hole neue ID:
	s.executeQuery ("select max(id) as m from artikel");
	results = s.getResultSet ();
	results.next();
	int newId = results.getInt("m") + 1;

	// Schreibe neuen Eintrag:
	s.execute ("insert into artikel values ("
		   + newId + ", \"" + title + "\")");
	
	// Finde nun alle Autoren:
	Pattern p1 = Pattern.compile("author *= *\\{");
	Matcher m1 = p1.matcher(bibtexEintrag);
	m1.find ();
	// Ok, Start-Position der Autoren gefunden...
    
	Pattern p2 = Pattern.compile("(.*?)( and |\\})");
	Matcher m2 = p2.matcher(bibtexEintrag);
	m2.find (m1.end()); // Suche ab dem Anfang.

	while (!m2.group(2).equals("}")) {
	  connectAuthorWithArticle (m2.group(1), newId);
	  m2.find();
	}
	connectAuthorWithArticle (m2.group(1), newId);
    }

    void connectAuthorWithArticle (String author, int article_id) throws Exception {
	int author_id;

	// Existiert Autor bereits?
	s.executeQuery ("select id from autor where name = \"" + author + "\"");
	results = s.getResultSet ();
	if (results.next()) {
	    author_id = results.getInt("id");
	} else {
	    // Hole neue ID:
	    s.executeQuery ("select max(id) as m from autor");
	    results = s.getResultSet ();
	    results.next();
	    author_id = results.getInt("m") + 1;
	    // Schreibe neuen Eintrag:
	    s.execute ("insert into autor values ("
		       + author_id + ", \"" + author + "\")");
	}

	// Schreibe die Verbindung in die Datenbank:
	s.execute ("insert into von values ("
		   + author_id + ", " + article_id + ")");
    }
    
    
    double computeImpactFactor (int zeitschriftId) throws Exception {
	s.executeQuery ("select count(*) as n from artikel, enthaelt" +
			" where artikel.id     = enthaelt.artikel_id and" +
			"       zeitschrift_id = " + zeitschriftId);
	results = s.getResultSet ();
	results.next();
	double n = results.getInt("n");
	
	s.executeQuery ("select count(*) as z from artikel, enthaelt, zitiert" +
			" where artikel.id     = enthaelt.artikel_id and" +
			"       zeitschrift_id = " + zeitschriftId + " and" +
			"       artikel.id     = zitiert.wird_zitiert_id");
	results = s.getResultSet ();
	results.next();
	double z = results.getInt("z");
	
	return z / n;
    }

    int computeHirschIndex (int autorId) throws Exception {
	s.executeQuery ("  select artikel.id, count(zitiert_id) as c " +
                            "from artikel, zitiert, von " +
                           "where artikel.id   = von.artikel_id and " +
                                 "artikel.id   = zitiert.wird_zitiert_id and " +
                                 "von.autor_id = " + autorId +
                       " group by artikel.id desc");
	results = s.getResultSet ();
	int hirsch = 0;

	while (results.next()) {
	    if (results.getInt("c") <= hirsch) {
		break;
	    }
	    else {
		hirsch++;
	    }
	}
	return hirsch;
    }
    
    public static void main (String[] args) {
	Datenbestand d = new Datenbestand ();

	try {
	    d.createConnection ("root", "");
	    d.readAllAuthors ();
	    d.readAllArticles ();
	    d.readAllJournals ();
	    d.readEnthaeltTable ();
	    //	    d.insertArticleIntoDatabse ("@book{Codd:1990:RMD:77708, author = {Johannes Textor and Till Tantau},  title = {Info A Skript},  year = {1990},  isbn = {0-201-14192-2},  publisher = {Addison-Wesley Longman Publishing Co., Inc.},  address = {Boston, MA, USA},}");
	    d.readAllArticles ();
	    System.out.println (d.computeHirschIndex (4));
	}
	catch (Exception e) {
	    e.printStackTrace ();
	}

    }
}
