public class SortingAlgorithms
{

  static String arrayToString (int[] array, int low, int high)
  {
    String return_me = "";
    for (int i = low; i <= high; i++)
      return_me = return_me + array[i] + " ";
    return return_me;
  }

  static String arrayToString (int[] array)
  {
    return arrayToString(array,0,array.length-1);
  }

  static void announce (String algorithm, int[] array)
  {
    System.out.println();
    System.out.println("Running algorithm " + algorithm + " on array " + arrayToString(array));
  }

  
  
  // Bubble Sort
  
  static void bubbleSort (int[] array)
  {
    announce("bubble sort", array);
    for (int hi = array.length-1; hi >= 0; hi = hi-1)
      for (int i = 0; i < hi; i = i+1)
        if (array[i] > array[i+1])
          swap(array, i, i+1);
   }

  static void swap (int[] array, int i, int j)
  {
    // Swaps elements array[i] and array[j]
    int temp = array[i];
    array[i] = array[j];
    array[j] = temp;

    System.out.println("Swapped positions " + i + " and " + j + " in array " + arrayToString(array));
  }
  
  
  // Selection Sort
  static void selectionSort (int[] array)
  {
    announce("selection sort", array);
    for (int pos = 0; pos < array.length-1; pos++)
    {
      // Find minimum following pos:
      int position_of_minimum = pos;

      for (int i = pos+1; i<array.length; i++)
        if (array[position_of_minimum] > array[i])
          position_of_minimum = i;

      swap(array,pos,position_of_minimum);
    }
  }

  
  
  // Insertion Sort
  
  static void insertionSort (int[] array)
  {
    announce("insertion sort", array);
    for (int pos = 0; pos < array.length; pos++)
    {
      int i = pos-1;
      while (i >= 0 && array[i] > array[i+1])
      {
        swap(array,i,i+1);
        i = i - 1;
      }
    }
  }

  
  
  
  // Merge Sort
  
  static void mergeSort (int[] array)
  { 
    announce("merge sort", array);
    if (array.length > 1)
    {
      // Copy first half of array to first
      int[] first  = new int [array.length/2];
      for (int i = 0; i<first.length; i++)
        first[i] = array[i];

      // Copy the rest (second half) of array to second
      int[] second = new int [array.length-first.length];
      for (int i = 0; i<second.length; i++)
        second[i] = array[i+first.length];

      System.out.println("Split " + arrayToString(array) + " into " 
                                  + arrayToString(first) + " and " 
                                  + arrayToString(second));

      // Sort first array
      mergeSort(first);

      // Sort second array
      mergeSort(second);

      // Merge sorted lists to a big sorted list
      merge(first,second,array);
 
      System.out.println("Merged " + arrayToString(first)  + " and " 
                                   + arrayToString(second) + " to "
                                   + arrayToString(array));
   }
  }

  static void merge(int[] first, int[] second, int[] array)
  {
    int first_pos  = 0;
    int second_pos = 0;
    int target_pos = 0;
          
    while (   first_pos  < first.length 
           || second_pos < second.length) 
    {
      if (first_pos < first.length &&
          (second_pos >= second.length ||
           first[first_pos] < second[second_pos]))
      { // Pick element from first
        array[target_pos] = first[first_pos];
        target_pos++;
        first_pos++;
      }
      else 
      { // Pick element from second
        array[target_pos] = second[second_pos];
        target_pos++;
        second_pos++;
      }
    } 
  }


  
  
  // Quicksort
  
  static void quickSort (int[] array, int start, int end)
  {
    announce("quick sort between " + start + " and " + end, array);
    if (start < end)
    {
      int pivot_pos = (start + end)/2;
      int pivot = array[pivot_pos];
      int end_of_left_side = start-1;

      swap (array, pivot_pos, end);
      for (int i = start; i < end; i++) 
        if (array[i] < pivot) 
        {
          end_of_left_side++;
          swap(array, end_of_left_side, i);
        }
      swap (array, end_of_left_side+1, end);        

      System.out.println ("Partioned array is " + arrayToString (array,start,end));

      quickSort(array,start,end_of_left_side);
      quickSort(array,end_of_left_side+2,end);
    }
  }

}
